//MemoryView.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//
#include <Point.h>
#include <OS.h>
#include <Debug.h>

#include <stdio.h>
#include <string.h>
#include "MemoryView.h"
#include "SysInfoDeltaGenerator.h"

static const char CLASS_NAME[] = "MemoryView";
static const char VIEW_NAME[] = "MemoryView";

static const BRect dummy(0.0f, 0.0f, 0.0f, 0.0f);
static const float MARGIN = 1.0f;
static const float STR_VIEW_HEIGHT = 9.0f;
static const float FONT_SIZE = 9.0f;
/* defines minimum&maximum size of the view */
static const BRect MIN_SIZE(0.0f, 0.0f, 60.0f, 20.0f);
static const BRect MAX_SIZE(0.0f, 0.0f, 1600.0f, 1200.0f);

// private observer class
class MemSysInfoObserver : public SysInfoObserver {
  public:
	MemSysInfoObserver(MemoryView &peer) : view(peer) {
	};
	
	~MemSysInfoObserver() {};
	
	inline void Updated(const SysInfo &delta) {
		float ar = (float)delta.GetInfo().used_pages /
					(float)delta.GetInfo().max_pages;

		view.Update(ar);
	};
	
  private:
	MemoryView &view;

	friend class MemoryView;
};

// Public: Constructor.
//
MemoryView::MemoryView(BRect frame)
	: BaseView(frame, VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS,
		   MIN_SIZE, MAX_SIZE) {
	// graph
	gView = new LineGraphView(frame, "Memory graph view", 1.0f);
	ASSERT(gView != NULL);
	AddChild(gView);

	// string
	sView = new LabelStringView(frame, "mem:", "0", "%");
	ASSERT(sView != NULL);
	AddChild(sView);
	
	// SysInfo observer
	obs = new MemSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Construcotr from the archive.
//
MemoryView::MemoryView(BMessage *archive)
	: BaseView(archive) {
	// restore transient pointer to the children
	//
	BView *child = this->ChildAt(0);

	while (child != NULL) {
		if (is_instance_of(child, LineGraphView))
			gView = cast_as(child, LineGraphView);
		else if (is_instance_of(child, LabelStringView))
			sView = cast_as(child, LabelStringView);
		
		child = child->NextSibling();
	}
	
	ASSERT(gView != NULL);
	ASSERT(sView != NULL);

	// SysInfo observer
	obs = new MemSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Destructor.
//
MemoryView::~MemoryView() {
	// Unregister SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Unregister(obs);
	delete(obs);
	obs = NULL;
}

// Public: Override BView method.
//
void
MemoryView::AttachedToWindow() {
	if (Parent()) {
		const rgb_color view = Parent()->ViewColor();
		SetViewColor(view);

		Adjust(Frame().Width(), Frame().Height());
	}
	
	// Register SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Register(obs);
	
	BaseView::AttachedToWindow();
}

// Public: Override BView method.
//
void
MemoryView::FrameResized(float width, float height) {
	Adjust(width, height);

	BaseView::FrameResized(width, height);
}

// Public: Update the contents.
//
void
MemoryView::Update(float rate) {
	char buf[1024];
	buf[0] = '\0';
	sprintf(buf, "%d", (int)(rate * 100.0f));
	sView->Update(buf);
	gView->Update(rate);
}

// Public: Arcive the object.
//
status_t
MemoryView::Archive(BMessage *archive, bool deep) const {
	status_t ret;
	ret = BaseView::Archive(archive, deep);
	ASSERT(ret == B_OK);
	
	ret = archive->AddString("class", CLASS_NAME);
	ASSERT(ret == B_OK);

	return B_OK;
}

// Public/Static: Instanciate the object.
//
BArchivable *
MemoryView::Instantiate(BMessage *archive) 
{
	if (validate_instantiation(archive, CLASS_NAME))
		return new MemoryView(archive); 
	
	return NULL; 
}

// Private:
//
MemoryView::MemoryView(const MemoryView &)
	: BaseView(dummy, VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW, MIN_SIZE, MAX_SIZE) {}

// Private: Adjust the size.
//
void
MemoryView::Adjust(float width, float height) {

	gView->MoveTo(MARGIN, MARGIN);
	gView->ResizeTo(width - MARGIN * 2.0f,
					height - STR_VIEW_HEIGHT - MARGIN);

	sView->MoveTo(MARGIN, height +MARGIN - STR_VIEW_HEIGHT);
	sView->ResizeTo(width - MARGIN * 2.0f, STR_VIEW_HEIGHT);
}

