// MeterBaseView.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//
#include <Point.h>
#include <OS.h>
#include <Messenger.h>
#include <stdio.h>
#include <string.h>
#include <Alert.h>
#include <Menu.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include <Debug.h>

#include "MeterMessages.h"
#include "BaseView.h"
#include "MeterBaseView.h"
#include "HorizontalLayoutManager.h"
#include "VerticalLayoutManager.h"
#include "CPUView.h"
#include "MemoryView.h"
#include "PageView.h"

static const char CLASS_NAME[] = "MeterBaseView";

static const char INTERVAL_NAME[] = "MeterBaseView::interval";
static const char ALL_WORKSPACES_NAME[] = "MeterBaseView::allWorkspaces";
static const char LAYOUT_NAME[] = "MeterBaseView::layout";
static const char VIEW_NAME_BASE[] = "BYap_MeterBaseView";

static const BMessage PING_MSG(BY_PING);
static const rgb_color VIEW_COLOR = {255, 255, 255, 0};

// Public: Constructor.
// Note: replicant hasn't been fully supported yet.
MeterBaseView::MeterBaseView(BRect frame, LayoutType type,
							 bool all, bigtime_t intvl)
	: BaseView(frame, VIEW_NAME_BASE, B_FOLLOW_ALL_SIDES,
	  B_FRAME_EVENTS /*| B_DRAW_ON_CHILDREN*/, BRect(), BRect()),
	  interval(intvl), allWorkspaces(all) {
	// Retrieve SysInfoDeltaGenerator.
	gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT((gen != NULL));

	// Create default LayoutManager.
	lm = LayoutManager::CreateLayoutManager(type, this);
	ASSERT((lm != NULL));
	
	// Create a CPU view.
	system_info info;
	get_system_info(&info);
	int cpuCount;
#ifdef DEBUG_CPU
	cpuCount = DEBUG_CPU;
#else
	cpuCount = info.cpu_count;
#endif
	BView *cpu;
	for (int i = 0; i < cpuCount; i++) {
		cpu = new CPUView(frame, i);
		AddChild(cpu);
	}

	// Create a memory view.
	BView *res = new MemoryView(frame);
	ASSERT((res != NULL));
	AddChild(res);
	
	// Create a page view
	res = new PageView(frame);
	ASSERT((res != NULL));
	AddChild(res);
}

// Public: Constructor from the archive.
//
MeterBaseView::MeterBaseView(BMessage *archive)
	: BaseView(archive) {
	// Retrieve SysInfoDeltaGenerator.
	gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	status_t ret;
	ret = archive->FindInt64(INTERVAL_NAME, &interval);
	ASSERT(ret == B_OK);
	ret = archive->FindBool(ALL_WORKSPACES_NAME, &allWorkspaces);
	ASSERT(ret == B_OK);
	int32 tmp;
	ret = archive->FindInt32(LAYOUT_NAME, &tmp);
	ASSERT(ret == B_OK);

	// Create LayoutManager.
	lm = LayoutManager::CreateLayoutManager((LayoutType)tmp, this);
	ASSERT((lm != NULL));
}

// Public: Destructor.
//
MeterBaseView::~MeterBaseView() {
	delete lm;
	lm = NULL;
}

// Public: Override BView method.
//
void
MeterBaseView::AllAttached() {
	TRACE();
	if (allWorkspaces)
		Window()->SetWorkspaces(B_ALL_WORKSPACES);

	// Set the size limits.
	lm->SetLimits();

	// Do the layout.
	lm->Layout();

	// Start MessageRunner.
	mr = new BMessageRunner(BMessenger(this), &PING_MSG, interval);
	ASSERT(mr->InitCheck() == B_OK);

	SetViewColor(VIEW_COLOR);
	BView::AllAttached();
}

// Public: Override BView method.
//
void
MeterBaseView::FrameResized(float width, float height) {
	TRACE();
	if (Parent() == NULL) // assume it's running as BeYap app.
		Window()->ResizeTo(Bounds().Width(), Bounds().Height());

	lm->Layout();
	
	BaseView::FrameResized(width, height);
}

// Public: Override BView method.
//
void
MeterBaseView::MessageReceived(BMessage *msg) {
	switch(msg->what) {
		case	B_ABOUT_REQUESTED:
		{
			BAlert *alert = new BAlert("About BeYap",
			"About BeYap\n"
			"Be Yet Another Performance meter\nVersion 0.2\n\n"
			"By Kiyoshi Kobayashi", "So so" B_UTF8_ELLIPSIS); 
			(void)alert->Go(NULL); // asynch
		}
			break;
		case	BY_PING:
			gen->Generate();
			break;
		case	BY_LY_VER:
			if (lm->GetLayoutType() != LT_VERTICAL) {
				delete lm;
				lm = LayoutManager::CreateLayoutManager(LT_VERTICAL, this);
			}
			lm->SetLimits();
			lm->Layout();
			break;
		case	BY_LY_HOR:
			if (lm->GetLayoutType() != LT_HORIZONTAL) {
				delete lm;
				lm = LayoutManager::CreateLayoutManager(LT_HORIZONTAL, this);
			}
			lm->SetLimits();
			lm->Layout();
			break;
		case	BY_LY_MAT:
			break;
		case	BY_ALL_WS:
			if (allWorkspaces) {
				allWorkspaces = false;
				Window()->SetWorkspaces(B_CURRENT_WORKSPACE);
			} else {
				allWorkspaces = true;
				Window()->SetWorkspaces(B_ALL_WORKSPACES);
			}
			break;
		default:
			BaseView::MessageReceived(msg);
			break;
	}
}

// Public: Override BView method.
//
void
MeterBaseView::AddMenuItems(BPopUpMenu *menu) {
	BMenu *l = new BMenu("Layout");
	
	// Horizontal
	BMenuItem *hor = new BMenuItem("Horizontal", new BMessage(BY_LY_HOR));
	if (lm->GetLayoutType() == LT_HORIZONTAL)
		hor->SetMarked(true);
	l->AddItem(hor);
	
	// Vertical
	BMenuItem *ver = new BMenuItem("Vertical", new BMessage(BY_LY_VER));
	if (lm->GetLayoutType() == LT_VERTICAL)
		ver->SetMarked(true);
	l->AddItem(ver);

	l->SetTargetForItems(this);
	menu->AddItem(l);

	// all workspace
	BMenuItem *aw = new BMenuItem("All Workspaces",
									new BMessage(BY_ALL_WS));
	if (allWorkspaces)
		aw->SetMarked(true);
	menu->AddItem(aw);

	// about
	BMenuItem *about = new BMenuItem("About" B_UTF8_ELLIPSIS,
									new BMessage(B_ABOUT_REQUESTED));
	

	menu->AddItem(about);

	menu->SetTargetForItems(this);
}

// Public: Archive the object.
//
status_t
MeterBaseView::Archive(BMessage *archive, bool deep) const {
	status_t ret;
	ret = BaseView::Archive(archive, deep);
	ASSERT(ret == B_OK);

	ret = archive->AddString("class", CLASS_NAME);
	ASSERT(ret == B_OK);
	ret = archive->AddString("add_on", "application/x-vnd.KYU-BeYap");
	ASSERT(ret == B_OK);

	if ( deep ) { 
		ret = archive->AddInt64(INTERVAL_NAME, interval);
		ASSERT(ret == B_OK);

		ret = archive->AddBool(ALL_WORKSPACES_NAME, allWorkspaces);
		ASSERT(ret == B_OK);

		ret = archive->AddInt32(LAYOUT_NAME, (int32)(lm->GetLayoutType()));
		ASSERT(ret == B_OK);
	}

	return B_OK;
}

// Public/Static: Instanciate the object.
//
BArchivable *
MeterBaseView::Instantiate(BMessage *archive) {
	if (validate_instantiation(archive, CLASS_NAME))
		return new MeterBaseView(archive); 
	
	return NULL; 
}

// Private:
//
MeterBaseView::MeterBaseView(const MeterBaseView &)
	: BaseView(BRect(), VIEW_NAME_BASE, B_FOLLOW_ALL, B_WILL_DRAW,
		   BRect(), BRect()) {
}

