// PageView.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//
#include <Point.h>
#include <OS.h>
#include <Debug.h>

#include <stdio.h>
#include <string.h>

#include "PageView.h"
#include "SysInfoDeltaGenerator.h"

static const char CLASS_NAME[] = "PageView";
static const char VIEW_NAME[] = "PageView";
static const char MAX_NAME[] = "PageView::max";

static const float MARGIN = 1.0f;
static const float STR_VIEW_HEIGHT = 9.0f;
static const float FONT_SIZE = 9.0f;


/* defines minimum&maximum size of the view */
static const BRect MIN_SIZE(0.0f, 0.0f, 60.0f, 20.0f);
static const BRect MAX_SIZE(0.0f, 0.0f, 1600.0f, 1200.0f);

// private observer class
class PageSysInfoObserver : public SysInfoObserver {
  public:
	PageSysInfoObserver(PageView &peer) : view(peer) {};
	~PageSysInfoObserver() {};
	
	inline void Updated(const SysInfo &delta) {
		float pr = (float)(delta.GetInfo().page_faults) /
				   (float)(delta.GetSystemTime());

		view.Update(pr);
	};
	
  private:
	PageView &view;

	friend class PageView;
};

// Public: Constructor.
//
PageView::PageView(BRect frame)
	: BaseView(frame, VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS,
		   MIN_SIZE, MAX_SIZE), max(1.0f) {
	// graph
	gView = new LineGraphView(frame, "Paging graph view");
	ASSERT(gView != NULL);
	AddChild(gView);

	// string
	sView = new BStringView(frame, "Paging string view", "page:",
							B_FOLLOW_ALL, B_WILL_DRAW);
	ASSERT(sView != NULL);
	AddChild(sView);

	// SysInfo observer
	obs = new PageSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Constructor from the archive.
//
PageView::PageView(BMessage *archive)
	: BaseView(archive) {
	status_t ret;
	ret = archive->FindFloat(MAX_NAME, &max);
	ASSERT(ret == B_OK);
	
	// restore transient pointer to the children
	//
	BView *child = this->ChildAt(0);

	while (child != NULL) {
		if (is_instance_of(child, LineGraphView))
			gView = cast_as(child, LineGraphView);
		else if (is_instance_of(child, BStringView))
			sView = cast_as(child, BStringView);
		
		child = child->NextSibling();
	}
	
	ASSERT(gView != NULL);
	ASSERT(sView != NULL);
	
	// SysInfo observer
	obs = new PageSysInfoObserver(*this);
	ASSERT(obs != NULL);
}

// Public: Destructor.
//
PageView::~PageView() {
	// Unregister SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Unregister(obs);
	delete(obs);
	obs = NULL;
}

// Public: Override BView method.
//
void
PageView::AttachedToWindow() {
	if (Parent()) {
		const rgb_color view = Parent()->ViewColor();
		SetViewColor(view);
		sView->SetViewColor(view);
	}
	sView->SetFontSize(FONT_SIZE);
	
	Adjust(Frame().Width(), Frame().Height());
	
	// Register SysInfo observer
	SysInfoDeltaGenerator *gen = SysInfoDeltaGenerator::GetGenerator();
	ASSERT(gen != NULL);
	
	gen->Register(obs);
	
	BaseView::AttachedToWindow();
}

// Public: Override BView method.
//
void
PageView::FrameResized(float width, float height) {
	Adjust(width, height);

	BaseView::FrameResized(width, height);
}

// Public: Update the contents.
//
void
PageView::Update(float rate) {
	gView->Update(rate);
}

// Public: Archive the object.
//
status_t
PageView::Archive(BMessage *archive, bool deep) const {
	status_t ret;
	ret = BaseView::Archive(archive, deep);
	ASSERT(ret == B_OK);
	
	ret = archive->AddString("class", CLASS_NAME);
	ASSERT(ret == B_OK);

	if ( deep ) { 
		ret = archive->AddFloat(MAX_NAME, max);
		ASSERT(ret == B_OK);
	}

	return B_OK;
}

// Public/Static: Instanciate the object.
//
BArchivable *
PageView::Instantiate(BMessage *archive) {
	if (validate_instantiation(archive, CLASS_NAME))
		return new PageView(archive);
	return NULL;
}

// Private:
//
PageView::PageView(const PageView &)
	: BaseView(BRect(), VIEW_NAME, B_FOLLOW_ALL, B_WILL_DRAW,
		   MIN_SIZE, MAX_SIZE), max(1.0) {}

// Private: Adjust the size.
//
void
PageView::Adjust(float width, float height) {
	gView->MoveTo(MARGIN, MARGIN);
	gView->ResizeTo(width - MARGIN * 2.0f,
					height - STR_VIEW_HEIGHT - MARGIN);

	sView->MoveTo(MARGIN, height + MARGIN - STR_VIEW_HEIGHT);
	sView->ResizeTo(width - MARGIN * 2.0f - 7.0f, STR_VIEW_HEIGHT);
}

