// Profile.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//

#include <Node.h>
#include <File.h>
#include <Directory.h>
#include <Debug.h>

#include "Profile.h"

static const char SETTING_DIR[] = "/boot/home/config/settings";
static const LayoutType DEFAULT_LAYOUT = LT_VERTICAL;
static const bool DEFAULT_ALL_WS = false;
static const bigtime_t DEFAULT_INTERVAL = 2000000;
static const BRect DEFAULT_FRAME(100.0f, 100.0f, 200.0f, 200.0f);

static const char FRAME_TOP[] = "Frame/top";
static const char FRAME_LEFT[] = "Frame/left";
static const char FRAME_BOTTOM[] = "Frame/bottom";
static const char FRAME_RIGHT[] = "Frame/right";
static const char LAYOUT[] = "LayoutType";
static const char ALL_WS[] = "AllWorkspaces";
static const char INTERVAL[] = "Interval";

Profile::Profile(const char *file)
  : filename(file), layout(DEFAULT_LAYOUT), all(DEFAULT_ALL_WS),
    interval(DEFAULT_INTERVAL) {
   	BDirectory dir(SETTING_DIR);
   	if ((stat = dir.InitCheck()) == B_OK) {
	  	BNode f(&dir, filename);
  		switch (stat = f.InitCheck()) {
  		case	B_NO_ERROR:
  			// Assume there is an profile file and try to use it.
  			if ((stat = ReadProfile(&f)) == B_NO_ERROR)
  				stat = B_OK;
  			break;
  		case	B_ENTRY_NOT_FOUND:
  			// Assume it's the first time to launch and create a file
  			// with defaults.
  			frame = DEFAULT_FRAME;
  			if ((stat = CreateDefaultProfile(&dir)) == B_NO_ERROR)
  				stat = B_OK;
  			break;
  		default:
  			stat = B_BAD_VALUE;
  			break;
  		}
  	}
}

Profile::~Profile() {
	stat = B_BAD_VALUE;
}

status_t
Profile::InitCheck() {
	return stat;
}

status_t
Profile::Commit() {
   	BDirectory dir(SETTING_DIR);
   	if ((stat = dir.InitCheck()) == B_OK) {
	  	BNode f(&dir, filename);
  		if ((stat = f.InitCheck()) == B_NO_ERROR) {
			if ((stat = WriteProfile(&f)) == B_NO_ERROR)
				stat = B_OK;
  		}
   	}
	ASSERT(stat == B_NO_ERROR);
   	return stat;
}

status_t
Profile::CreateDefaultProfile(BDirectory *dir) {
	status_t ret = B_NO_ERROR;
	BFile file;
	if ((ret = dir->CreateFile(filename, &file, true)) == B_OK) {
		BNode n(dir, filename);
		ret = WriteProfile(&n);
	}
	ASSERT(ret == B_NO_ERROR);
	return ret;
}

status_t
Profile::ReadProfile(BNode *n) {
	int num;
	status_t ret = n->Lock();
	ASSERT(ret == B_NO_ERROR);
	num = n->ReadAttr(FRAME_TOP, B_FLOAT_TYPE, 0,
					(void *)&frame.top, sizeof(float));
	ASSERT(num > 0);
	num = n->ReadAttr(FRAME_LEFT, B_FLOAT_TYPE, 0,
					(void *)&frame.left, sizeof(float));
	ASSERT(num > 0);
	num = n->ReadAttr(FRAME_BOTTOM, B_FLOAT_TYPE, 0,
					(void *)&frame.bottom, sizeof(float));
	ASSERT(num > 0);
	num = n->ReadAttr(FRAME_RIGHT, B_FLOAT_TYPE, 0,
					(void *)&frame.right, sizeof(float));
	ASSERT(num > 0);
	num = n->ReadAttr(LAYOUT, B_INT32_TYPE, 0,
					(void *)&layout, sizeof(int32));
	ASSERT(num > 0);
	num = n->ReadAttr(ALL_WS, B_BOOL_TYPE, 0,
					(void *)&all, sizeof(bool));
	ASSERT(num > 0);
	num = n->ReadAttr(INTERVAL, B_INT64_TYPE, 0,
					(void *)&interval, sizeof(uint64));
	ASSERT(num > 0);
	ret = n->Unlock();
	ASSERT(ret == B_NO_ERROR);
	if (num < 0)
		ret = num;
	return ret;
}

status_t
Profile::WriteProfile(BNode *n) {
	int num;
	status_t ret = n->Lock();
	ASSERT(ret == B_NO_ERROR);
	num = n->WriteAttr(FRAME_TOP, B_FLOAT_TYPE, 0,
					(void *)&frame.top, sizeof(float));
	ASSERT(num > 0);
	num = n->WriteAttr(FRAME_LEFT, B_FLOAT_TYPE, 0,
					(void *)&frame.left, sizeof(float));
	ASSERT(num > 0);
	num = n->WriteAttr(FRAME_BOTTOM, B_FLOAT_TYPE, 0,
					(void *)&frame.bottom, sizeof(float));
	ASSERT(num > 0);
	num = n->WriteAttr(FRAME_RIGHT, B_FLOAT_TYPE, 0,
					(void *)&frame.right, sizeof(float));
	ASSERT(num > 0);
	num = n->WriteAttr(LAYOUT, B_INT32_TYPE, 0,
					(void *)&layout, sizeof(int32));
	ASSERT(num > 0);
	num = n->WriteAttr(ALL_WS, B_BOOL_TYPE, 0,
					(void *)&all, sizeof(bool));
	ASSERT(num > 0);
	num = n->WriteAttr(INTERVAL, B_INT64_TYPE, 0,
					(void *)&interval, sizeof(uint64));
	ASSERT(num > 0);
	ret = n->Unlock();
	ASSERT(ret == B_NO_ERROR);
	if (num < 0)
		ret = num;
	return ret;
}
